#!/bin/bash
############################################################################
# FolderByGroup 1.0 by Turranius  ( http://www.grandis.nu/glftpd/ )        #
####                                                                    ####
# This script is ment for dated folders. Mostly MP3 but it will work for   #
# 0days too. Its ment to be run at midnight (or a few seconds after to be  #
# sure). It will then go through yesterdays dir where it will              #
# sort the releases into folders based on the groups that released it.     #
# Example Before:                                                          #
# /glftpd/site/MP3/0707/Some-Lame-_-CountrySong-WeROCK                     #
# After:                                                                   #
# /glftpd/site/MP3/0707/WeROCK/Some-Lame-_-CountrySong-WeROCK              #
#                                                                          #
# It can also remove any nuked folders before doing this, as well as       #
# created a new dated dir for today, setting permissions and owner on both #
# the yesterday and the today folder.                                      #
# Of course, it can also create symlink for todays and yesterdays dir      #
# anywhere you tell it to.                                                 #
#                                                                          #
# NOTE: Running this script with the parameter 'test' like                 #
# folderbygroup.sh test                                                    #
# will show you onscreen what it would have done if you did not say test :)#
# It will say it does some silly things, like move nuked folders even      #
# though it said it already deleted them, and say it creates the groups    #
# folders multiple times, but thats just because it never actually does it.#
#                                                                          #
# Use this atleast the first time after you set it up to verify the setup. #
#--------------------------------------------------------------------------#
# Installation:
# Copy this script anywhere you like (/glftpd/bin?) and make it executable.#
# Change the setting as follows:                                           #
#                                                                          #
# LOG=         To what file do you want it to log?                         #
# FROM=        The parent folder of you dated dir to check. (/site/MP3?)   #
#                                                                          #
# FORCE=       The script will check the group name based on the name it   #
#              finds after the last "-" in the foldername. Sometimes this  #
#              might not be correct. Especially if the group has a - in    #
#              its name. This setting allows you to force something it     #
#              finds into another folder.                                  #
#              So, if it moves the groupname We-Rock into the folder Rock, #
#              you would add Rock:We-Rock here. Separate forces with a     #
#              space, like in the example.                                 #
#              It can also be used if you want some groups releases into   #
#              a special folder.. like RapRUs:CRAP_RAP would move any      #
#              release from RapRUS into the CRAP_RAP folder.               #
#              Setting it to "" will disable this function.                #
#                                                                          #
# EXCLUDE=     Anything you want excluded from moving.                     #
#              The default is ".message|\[incomplete\]" because I do not   #
#              want it to sort .message files or folders containing        #
#              [incomplete] (symlinks created by Dark0n3s Zipscriptp-c).   #
#              Separate excludes with a | and dont forget to escape        #
#              special characters like [ or } to \[ or \}                  #
#                                                                          #
# MOVEFOLDER=  If TRUE, it will move the folder into its new group folder. #
#              If FALSE, it will just copy them. I dont see why anyone     #
#              would only like to copy them, but I included it anyway.     #  
#                                                                          #
# DELNUKES=    Shall the script delete all the nuked folders it finds?     #
#              If FALSE, nuked folders will be sorted too.                 #
# NUKENAME=    What does your nuked folders name contain that other        #
#              folders does not? Everything with this in its name will be  #
#              deleted if DELNUKES is TRUE. As default, its just NUKED-    #
#                                                                          #
# If all you want is for this script for sort the folders like this, set   #
# MAKENEXTDAY and CREATESYMLINKS below here to FALSE, and it wont do       #
# anything else. If not:                                                   #
#                                                                          #
# MAKENEXTDAY=  Shall the script create the next days dated folder?        #
#               TRUE/FALSE                                                 #
#                                                                          #
# NEWCHMOD=     What shall the script chmod the new folder to?             #
#               Setting it to "" will disable this function.               #
# NEWOWNER=     What shall the script change the new folder's owner too?   #
#               Setting it to "" will disable this function.               #
# OLDCHMOD=     What shall this script chmod yesterdays folder too?        #
#               Good if you do not want people to upload files into an old #
#               folder. Set it to "" to disable this function.             #
#                                                                          #
# CREATESYMLINK=    Shall the script create a symlink somewhere for the    #
#                   new dated folder? TRUE/FALSE.                          #
# CREATEOLDSYMLINK= Shall the script create a symlink to yesterdays dated  #
#                   folder? TRUE/FALSE.                                    #
# FROMCHROOT=       What is the sites /site folder called from glftpd's    #
#                   perspective? Usually just /site                        #
# WHERE=            Where to put these symlinks? Not chrooted.             #
# NEWNAME=          What do we call the symlink pointing to todays folder? #
#                   (Mp3Today perhaps?)                                    #
# OLDNAME=          What do we call the symlink pointing to yesterdays     #
#                   folder? (Mp3Yesterday perhaps?)                        #
#                                                                          #
# today=            The structure of you TODAY folder. Its default is      #
#                   0503 for the 3rd of May.                               #
# yesterday=        Same as above, but for yesterdays folder.              #
#                                                                          #
####--------------------------------------------------------------------####
# Changelog:                                                               #
# 1.0 : Initial release.                                                   #
############################################################################
# Contact Turranius on efnet (turranius/turran/turr|away/turr|work).       #
# Usually Turran in #glftpd                                                #
############################################################################

LOG="/glftpd/ftp-data/logs/folderbygroup.log"  # What to log to.
FROM="/glftpd/site/MP3"                        # Parent to the dated folder.

FORCE="NuHS:NUHS pyt:SomethingElse"            # Replace what with what? Space separated.

EXCLUDE='.message|\[incomplete\]'              # Any folders or files to ignore? '|' separated.
MOVEFOLDERS="TRUE"                             # TRUE = Move them. FALSE = Copy them.

DELNUKES="TRUE"                                # Delete nuked folders instead of sorting them too?
NUKENAME="NUKED-"                              # What does your nuked foldernames contain that others do not?

MAKENEXTDAY="TRUE"                             # Create folders for today?
NEWCHMOD="777"                                 # If above is TRUE, what permissions to set on it? ("" = Dont)
NEWOWNER="0:0"                                 # And, who should own the folder? ("" = Dont)
OLDCHMOD="755"                                 # What to chmod yesterdays folder to? ("" = Dont) 

CREATESYMLINK="TRUE"                           # Create a symlink to the new dir today?
CREATEOLDSYMLINK="TRUE"                        # Create a symlink to yesterdays dir?
FROMCHROOT="/site"                             # Chrooted version of FROM= (like glftpd sees it).
WHERE="/glftpd/site"                           # Where to put the symlink(s)?
NEWNAME="MP3Today"                             # Todays symlink name.
OLDNAME="MP3Yesterday"                         # Yesterdays symlink name.

today="$( date +%m%d )"                        # Dated dir structure.
yesterday="$( date --date "yesterday" +%m%d )" # Yesterdays Structure.


##################################################################################
## No changes below needed, or.. heck do I care.. Its your system.               #
##################################################################################

## Just incase someone runs it at midnight exactly.
if [ "$1" != "test" ]; then
  sleep 2
fi

## For the log..
DATENOW="$(date +%D" - "%T)"

if [ "$MOVEFOLDERS" = "TRUE" ]; then
  CMD="mv -f"
else
  CMD="cp -Rf"
fi

if [ "$1" != "test" ]; then
  echo " " >> $LOG
  echo "---------------------------------------------------------------------------------" >> $LOG
  echo "$DATENOW : -- Starting FolderByGroup script. Today=$today. Yesterday=$yesterday" >> $LOG
fi

ERROR=""
toexists="$FROM"
if [ -e "$toexists" ]; then
  found="yes"
else
  if [ "$1" = "test" ]; then
    echo "Can not find path $toexists."
  else
    echo "$DATENOW : Can not find path $toexists. Check settings in FROM" >> $LOG
  fi
  ERROR="TRUE"
fi

if [ "$ERROR" = "TRUE" ]; then
  exit 0
fi

dir="$FROM"

## Create a new folder for today, if on.
if [ "$MAKENEXTDAY" = "TRUE" ]; then
  if [ -e "$dir/$today" ]; then
    if [ "$1" = "test" ]; then
      echo "$today folder already found. Not creating it."
    else
      echo "$DATENOW : $today folder already found. Not creating it." >> $LOG
    fi
  else
    if [ "$1" = "test" ]; then
      echo "Creating todays folder $dir/$today."
    else
      echo "$DATENOW : Creating todays folder $dir/$today." >> $LOG
      mkdir $dir/$today
      if [ "$NEWCHMOD" != "" ]; then
        chmod $NEWCHMOD $dir/$today
      fi
      if [ "$NEWOWNER" != "" ]; then
        chown $NEWOWNER $dir/$today
      fi
      if [ "$OLDCHMOD" != "" ]; then
        chmod $OLDCHMOD $dir/$yesterday
      fi
    fi
  fi
   
  ## Create symlinks.
  if [ "$CREATESYMLINK" = "TRUE" ]; then
    cd $WHERE
    if [ "$1" = "test" ]; then
      echo "Creating symlink: ln -f -s $FROMCHROOT/$today '$WHERE/$NEWNAME'"
    else
       echo "Creating symlink: ln -f -s $FROMCHROOT/$today '$WHERE/$NEWNAME'" >> $LOG
       ln -f -s $FROMCHROOT/$today "$WHERE/$NEWNAME"
    fi
    if [ "$CREATEOLDSYMLINK" = "TRUE" ]; then
      if [ "$1" = "test" ]; then
        echo "Creating yesterdays symlink: ln -f -s $FROMCHROOT/$yesterday '$WHERE/$OLDNAME'"
      else
        echo "Creating yesterdays symlink: ln -f -s $FROMCHROOT/$yesterday '$WHERE/$OLDNAME'" >> $LOG
        ln -f -s $FROMCHROOT/$yesterday "$WHERE/$OLDNAME"
      fi
    fi
  fi
fi

## Delete nukes, if on.
if [ "$DELNUKES" = "TRUE" ]; then
  cd $dir/$yesterday
  for nukedfolder in `ls -f -A | grep $NUKENAME`; do
    if [ "$1" = "test" ]; then
      echo "Deleting nuked folder $nukedfolder"
    else
      rm -rf $nukedfolder
      echo "Deleting nuked folder $nukedfolder" >> $LOG
    fi
  done
fi

## Start moving!
if [ -e "$dir/$yesterday" ]; then
  cd $dir/$yesterday
  for i in `ls -f -A | egrep -v $EXCLUDE`; do
    GROUPTEST="$( echo $i | tr -s '-' ' ' )"
    for field in $GROUPTEST; do
      GROUPCHECK="$field"
    done

    ## Check forced folders..
    if [ "$FORCE" != "" ]; then
      for forced in $FORCE; do
        FNAME="$( echo $forced | awk -F":" '{print $1}' )"
        if [ "$FNAME" != "" ]; then
          if [ "$FNAME" = "$GROUPCHECK" ]; then
            FNEWNAME="$( echo $forced | awk -F":" '{print $2}' )"
            GROUPCHECK="$FNEWNAME"
          fi
        fi
      done
    fi

    if [ -e "$dir/$yesterday/$GROUPCHECK" ]; then
      already="true"
    else
      if [ "$1" = "test" ]; then
        if [ "$LAST" != "$GROUPCHECK" ]; then
          echo "Mkdir $dir/$yesterday/$GROUPCHECK"
          LAST="$GROUPCHECK"
        fi
      else
        mkdir $dir/$yesterday/$GROUPCHECK
        echo "$DATENOW : Mkdir $dir/$yesterday/$GROUPCHECK" >> $LOG
      fi
    fi
    if [ "$1" = "test" ]; then
      echo "$CMD $dir/$yesterday/$i $dir/$yesterday/$GROUPCHECK"
    else
      echo "$DATENOW : $CMD $yesterday/$i $dir/$yesterday/$GROUPCHECK" >> $LOG
      if [ -e "$dir/$yesterday/$GROUPCHECK" ]; then
        $CMD $dir/$yesterday/$i $dir/$yesterday/$GROUPCHECK
      else
        echo "$DATENOW : ERROR. Could not create $dir/$yesterday/$GROUPCHECK" >> $LOG
        echo "$DATENOW : ERROR. I should have done that, but was apparently not able too. Check perms." >> $LOG
      fi
    fi 

  done
else
  if [ "$1" = "test" ]; then
    echo "Can not find $dir/$yesterday to check in."     
  else
    echo "$DATENOW : Could not find $dir/$yesterday to check in. Not a bad thing if you have multiple FROM sections." >> $LOG
  fi
fi

exit 0
